#
# d-SCORE
#

if(!require(plotrix))
	install.packages("plotrix", repos="http://software.rc.fas.harvard.edu/mirrors/R/")
library(plotrix)

####### ---------------------------- #######
#######       READ GCT FILE          #######
####### ---------------------------- #######

read.gct <- function(datafile="") {
	if(datafile=="") {
		datafile = choose.files(multi=F, filter=c("GCT file","*.gct"))
		if(length(datafile)==0)
			return()
	}	
	x = read.table(datafile, as.is=T, skip=2, header=T, sep="\t", quote="\"", check.names=F, fill=TRUE, comment.char="")
	rownames(x) = as.character(x[,1])
	return(x)
}

####### ---------------------------- #######
#######       READ CLS FILE          #######
####### ---------------------------- #######

read.cls <- function(clsFile="") {
	if(clsFile=="") {
		clsFile = choose.files(multi=F, filter=c("CLS file","*.cls"))
		if(length(clsFile)==0)
			return()
	}	
	x = list()
	classes	   = 1+c(-1,-1, scan(file=clsFile, sep=" ", what=integer(0), skip=2, quiet=T))
	numClasses = max(classes)
	classNames = scan(file=clsFile, sep=" ", what=character(0), nmax=numClasses+1, skip=1, quiet=T)[-1]
	x$classNames = classNames
	x$membership = classes	
	return(x)
}

####### ---------------------------- #######
####### CREATE USER FRIENDLY LABELS  #######
####### ---------------------------- #######

fc.labels <- function(x) {
	return(c(paste("1",  2^abs(x[x<0]), sep="/"), 2^x[x>=0]))
}

####### ---------------------------- #######
#######   GENERATE COLOR GRADIENT    #######
####### ---------------------------- #######

colorGradient <- function(N, colorKeys) {
	if(length(colorKeys)==1) {
		if(colorKeys=="Midnight") {
			colorKeys = c(rgb(0,0,0), rgb(0,0,1), rgb(1,1,1))
		} else if(colorKeys=="Royal") {
			colorKeys = c(rgb(0,0,1), rgb(0,0,0), rgb(1,0,0)) 
		} else if(colorKeys=="Hot") {
			colorKeys = c(rgb(0,0,0), rgb(1,0,0), rgb(1,1,0), rgb(1,1,1)) 
		} else if(colorKeys=="Sunset") {
			colorKeys = c(rgb(0,0,0), rgb(0,0,1), rgb(1,0,0), rgb(1,1,0)) 
		} else if(colorKeys=="Africa") {
			colorKeys = c(rgb(0,1,0), rgb(0,0,0), rgb(1,0,0)) 
		} else if(colorKeys=="GenePattern") {
			colorKeys = c(rgb(0,0,1), rgb(1,1,1), rgb(1,0,0))
		}
	} 
	index	 = 0
	gradient = array(,N) 
	numKeys	 = length(colorKeys)
	keys	 = t(col2rgb(colorKeys)/255)
	for(n in 1:(numKeys-1)) {
		block = round((N-index)/(numKeys-n))
		delta = (keys[n+1,]-keys[n,])/block
		for(i in 0:(block-2)) {
			index = index+1
			shade = keys[n,]+i*delta 
			gradient[index] = rgb(shade[1], shade[2], shade[3])		
		}
	}
	shade = keys[numKeys,]
	gradient[N] = rgb(shade[1], shade[2], shade[3])
	return(gradient)
}


####### ---------------------------- #######
#######            MAIN              #######
####### ---------------------------- #######

d.score <- function(gctFile, clsFile, comparisons, stabilityCoefficient=2) {
	Vc = as.integer(stabilityCoefficient)
	dat = read.gct(gctFile)
	cls = read.cls(clsFile)
	populations = read.table(comparisons, sep="\t", as.is=T, header=F, comment.char="", quote='\"')

	allPopulations = unlist(populations)
	missing = array(,0)
	for(i in 1:length(allPopulations)) {
		if(!(allPopulations[i] %in% cls$classNames)) {
			missing = c(missing, allPopulations[i])
		}
	}
	if(length(missing)>0) {
		cat("ERROR: Some of the populations listed in the comparison file are not found in the CLS file. Please check your file for errors.")
		stop()
	}

	N = 512
	xTicks = seq(from=-5, to=5, by=1)
	colorScale = colorGradient(N, "Sunset")
	fc = seq(from=0,to=5,by=0.1)
	dScores = seq(from=0.5, to=5, by=0.5)

	for(i in 1:nrow(populations)) {
		popA = cls$membership==(which(cls$classNames == populations[i,1]))
		popB = cls$membership==(which(cls$classNames == populations[i,2]))
		dataA = log2(dat[, popA])
		dataB = log2(dat[, popB])
		meanA = rowMeans(dataA)
		meanB = rowMeans(dataB)
		cvA = apply(dat[, popA], MARGIN=1, FUN=sd)/rowMeans(dat[, popA])	# Compute the incorrect CV
		cvB = apply(dat[, popB], MARGIN=1, FUN=sd)/rowMeans(dat[, popB])
		CV = (cvA+cvB)/2
		lDiff = meanA-meanB   # same as: log2(2^meanA/2^meanB)
		SD = apply(dataA, MARGIN=1, FUN=sd) + apply(dataB, MARGIN=1, FUN=sd)

		delta = abs(lDiff) - abs(SD^Vc)
		delta[delta<0] = 0
		delta[delta>5] = 5
		#Q = round(1+(N-1)*Q/5)

		baseName = paste(populations[i,1], "vs", populations[i,2])
		png(filename=paste(baseName, "- pineapple (FC x CV).png"), width=1400, height=500) 
		layout(matrix(1:2,1,2))
		par(mai=c(1,0.9,0.9,0.8), xaxs="i", yaxs="i")

		plot(lDiff[order(delta)], cvA[order(delta)], pch =20, las=1, xaxt="n", cex=1.0, cex.axis=0.8, xlab="Fold Change", ylab=paste("CV of",populations[i,1]), main="", xlim=range(xTicks), ylim=c(0,1.5), col=colorScale[sort(round(1+(N-1)*delta/5))])
		axis(side=1, at=xTicks, labels=fc.labels(xTicks), cex.axis=0.8)
		title(main=paste(populations[i,1]," vs ", populations[i,2], "\n", sep=""), line=1)
		title(main=bquote(paste(delta, " score")), line=1.5)
		color.legend(4.5,0.75,5,1.5, legend=paste("- ",format(c(0,dScores), nsmall=1), c("","+")[1+(c(0,dScores)==5)], sep=""), rect.col=rev(colorScale), cex=0.7, align="rb", gradient="y", font=2)
		
		plot(lDiff[order(delta)], cvB[order(delta)], pch =20, las=1, xaxt="n", cex=1.0, cex.axis=0.8, xlab="Fold Change", ylab=paste("CV of",populations[i,2]), main="", xlim=range(xTicks), ylim=c(0,1.5), col=colorScale[sort(round(1+(N-1)*delta/5))])
		axis(side=1, at=xTicks, labels=fc.labels(xTicks), cex.axis=0.8)
		title(main=paste(populations[i,1]," vs ", populations[i,2], "\n", sep=""), line=1)
		title(main=bquote(paste(delta, " score")), line=1.5)
		color.legend(4.5,0.75,5,1.5, legend=paste("- ",format(c(0,dScores), nsmall=1), c("","+")[1+(c(0,dScores)==5)], sep=""), rect.col=rev(colorScale), cex=0.7, align="rb", gradient="y", font=2)

		dev.off()

# 		png(filename=paste(baseName, "- pineapple 2 (FC x gSD).png"), width=700, height=500) 
# 		#layout(matrix(1:2,1,2))
# 		par(mai=c(1,0.9,0.9,0.8), xaxs="i", yaxs="i")
# 
# 		plot(lDiff[order(delta)], SD[order(delta)], pch =20, las=1, xaxt="n", cex=1.3, cex.axis=0.8, xlab="Fold Change", ylab="Total Geometric Standard Deviation", xlim=range(xTicks), ylim=c(0,2.5), col=colorScale[sort(round(1+(N-1)*delta/5))])
# 		axis(side=1, at=xTicks, labels=fc.labels(xTicks), cex.axis=0.8)
# 		title(main=paste(populations[i,1]," vs ", populations[i,2], "\n", sep=""), line=1)
# 		title(main=bquote(paste(delta, " score")), line=1.5)
# 		d2 = dScores[-10]
# 		axis(side=4, at=(max(xTicks)-d2)^(1*(Vc^-1)), labels=paste("d =", format(d2, nsmall=1)), col=colorScale[round(1+(N-1)*d2/5)], line=-0.5, las=2, cex.axis=0.7, font=2, tick=F)
# 		#for(d in d2)
# 		#	axis(side=4, at=(max(xTicks)-d)^(1*(Vc^-1)), labels=bquote(paste(delta==.(format(d, nsmall=1)))), col=colorScale[round(1+(N-1)*d/5)], line=-0.5, las=2, cex.axis=0.7, font=2, tick=F)
# 		for(d in dScores) {
# 			sdD = (fc-d)^(1/Vc)
# 			color = colorScale[round(1+(N-1)*d/5)]
# 			lineSize = 1
# 			par(new=T);
# 			plot(fc, sdD, type="l", lwd=lineSize, xaxt="n", yaxt="n", xlab="", ylab="", main="", xlim=range(xTicks), ylim=c(0,2.5), col=color)
# 			par(new=T);
# 			plot(-fc, sdD, type="l", lwd=lineSize, xaxt="n", yaxt="n", xlab="", ylab="", main="", xlim=range(xTicks), ylim=c(0,2.5), col=color)
# 		}
# 		dev.off()
		
		for(d in dScores) {
			write.table(names(delta[delta>=d]), paste(baseName, " (delta above ", format(d, nsmall=1), ").txt", sep=""), col.names=F, row.names=F, quote=F)
		}
	}
}




